#!/usr/bin/env python3
"""
Windows System Information Viewer (System Management Tool)

Parameters:
    --overview: Show system overview
    --cpu: Show CPU information
    --memory: Show memory information
    --os: Show OS information
    --hardware: Show hardware details
    --drivers: List installed drivers
    --updates: Show Windows updates
    --format FORMAT: Output format (table/json)

Requires: psutil, win32api, tabulate packages
Warning: Some information may require admin privileges to access
"""

import os
import sys
import platform
import json
import argparse
import logging
from typing import List, Dict, Any, Optional
from tabulate import tabulate
import wmi
import win32api
import win32security
import win32net
import win32netcon
import winreg
import psutil
from datetime import datetime
import time

class SystemInformationViewer:
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self.wmi = wmi.WMI()

    def get_system_info(self) -> Dict[str, Any]:
        """Get comprehensive system information."""
        return {
            'hardware': self.get_hardware_info(),
            'os': self.get_os_info(),
            'software': self.get_installed_software(),
            'drivers': self.get_driver_info(),
            'security': self.get_security_info(),
            'network': self.get_network_info(),
            'users': self.get_user_info()
        }

    def get_hardware_info(self) -> Dict[str, Any]:
        """Get detailed hardware information."""
        hardware = {
            'processor': self._get_processor_info(),
            'memory': self._get_memory_info(),
            'disks': self._get_disk_info(),
            'motherboard': self._get_motherboard_info(),
            'graphics': self._get_graphics_info(),
            'bios': self._get_bios_info()
        }
        return hardware

    def get_os_info(self) -> Dict[str, Any]:
        """Get operating system information."""
        os_info = {
            'name': platform.system(),
            'version': platform.version(),
            'architecture': platform.machine(),
            'build': platform.win32_ver(),
            'hostname': platform.node(),
            'boot_time': datetime.fromtimestamp(
                psutil.boot_time()
            ).strftime('%Y-%m-%d %H:%M:%S'),
            'uptime': self._format_uptime(psutil.boot_time())
        }
        
        try:
            os_info.update(self._get_windows_info())
        except Exception as e:
            self.logger.error(f"Error getting Windows info: {str(e)}")
            
        return os_info

    def get_installed_software(self) -> List[Dict[str, Any]]:
        """Get list of installed software."""
        software_list = []
        
        def get_reg_value(key, name):
            try:
                return winreg.QueryValueEx(key, name)[0]
            except:
                return None
        
        for hive in [winreg.HKEY_LOCAL_MACHINE, winreg.HKEY_CURRENT_USER]:
            for key_path in [
                r"Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall",
                r"Software\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion"
                r"\\Uninstall"
            ]:
                try:
                    key = winreg.OpenKey(hive, key_path)
                    for i in range(winreg.QueryInfoKey(key)[0]):
                        try:
                            subkey_name = winreg.EnumKey(key, i)
                            subkey = winreg.OpenKey(key, subkey_name)
                            
                            name = get_reg_value(subkey, "DisplayName")
                            if name:  # Only add if it has a display name
                                software = {
                                    'name': name,
                                    'version': get_reg_value(
                                        subkey, "DisplayVersion"
                                    ),
                                    'publisher': get_reg_value(
                                        subkey, "Publisher"
                                    ),
                                    'install_date': get_reg_value(
                                        subkey, "InstallDate"
                                    ),
                                    'install_location': get_reg_value(
                                        subkey, "InstallLocation"
                                    )
                                }
                                software_list.append(software)
                                
                        except WindowsError:
                            continue
                            
                except WindowsError:
                    continue
                    
        return sorted(software_list, key=lambda x: x['name'].lower())

    def get_driver_info(self) -> List[Dict[str, Any]]:
        """Get system driver information."""
        drivers = []
        for driver in self.wmi.Win32_SystemDriver():
            try:
                drivers.append({
                    'name': driver.Name,
                    'display_name': driver.DisplayName,
                    'path': driver.PathName,
                    'state': driver.State,
                    'start_mode': driver.StartMode,
                    'service_type': driver.ServiceType
                })
            except Exception as e:
                self.logger.debug(f"Error getting driver info: {str(e)}")
                continue
        return drivers

    def get_security_info(self) -> Dict[str, Any]:
        """Get system security information."""
        security = {
            'uac': self._get_uac_settings(),
            'firewall': self._get_firewall_settings(),
            'antivirus': self._get_antivirus_info(),
            'updates': self._get_windows_updates()
        }
        return security

    def get_network_info(self) -> Dict[str, Any]:
        """Get network configuration information."""
        network = {
            'hostname': platform.node(),
            'domain': win32api.GetComputerNameEx(
                win32con.ComputerNameDnsDomain
            ),
            'interfaces': self._get_network_interfaces(),
            'dns': self._get_dns_servers(),
            'shares': self._get_network_shares()
        }
        return network

    def get_user_info(self) -> List[Dict[str, Any]]:
        """Get user account information."""
        users = []
        resume = 0
        while True:
            try:
                user_list, total, resume = win32net.NetUserEnum(
                    None,
                    2,
                    win32netcon.FILTER_NORMAL_ACCOUNT,
                    resume,
                    win32netcon.MAX_PREFERRED_LENGTH
                )
                for user in user_list:
                    try:
                        groups = win32net.NetUserGetLocalGroups(None, user['name'])
                        users.append({
                            'name': user['name'],
                            'full_name': user['full_name'],
                            'comment': user['comment'],
                            'flags': user['flags'],
                            'priv': user['priv'],
                            'home_dir': user['home_dir'],
                            'last_logon': user['last_logon'],
                            'groups': groups
                        })
                    except win32net.error:
                        continue
                if not resume:
                    break
            except win32net.error:
                break
        return users

    def _get_processor_info(self) -> Dict[str, Any]:
        """Get detailed processor information."""
        cpu = self.wmi.Win32_Processor()[0]
        return {
            'name': cpu.Name,
            'manufacturer': cpu.Manufacturer,
            'cores': psutil.cpu_count(logical=False),
            'threads': psutil.cpu_count(logical=True),
            'max_speed': cpu.MaxClockSpeed,
            'voltage': cpu.CurrentVoltage,
            'l2_cache': cpu.L2CacheSize,
            'l3_cache': cpu.L3CacheSize,
            'architecture': cpu.Architecture,
            'socket': cpu.SocketDesignation
        }

    def _get_memory_info(self) -> Dict[str, Any]:
        """Get detailed memory information."""
        virtual = psutil.virtual_memory()
        swap = psutil.swap_memory()
        
        physical_memory = []
        for mem in self.wmi.Win32_PhysicalMemory():
            physical_memory.append({
                'manufacturer': mem.Manufacturer,
                'part_number': mem.PartNumber,
                'capacity': mem.Capacity,
                'speed': mem.Speed,
                'form_factor': mem.FormFactor,
                'location': mem.DeviceLocator
            })
            
        return {
            'total': virtual.total,
            'available': virtual.available,
            'used': virtual.used,
            'physical_memory': physical_memory,
            'swap': {
                'total': swap.total,
                'used': swap.used,
                'free': swap.free
            }
        }

    def _get_disk_info(self) -> List[Dict[str, Any]]:
        """Get detailed disk information."""
        disks = []
        for disk in self.wmi.Win32_DiskDrive():
            try:
                disks.append({
                    'name': disk.Caption,
                    'size': disk.Size,
                    'interface': disk.InterfaceType,
                    'partitions': disk.Partitions,
                    'model': disk.Model,
                    'serial': disk.SerialNumber,
                    'firmware': disk.FirmwareRevision
                })
            except Exception as e:
                self.logger.debug(f"Error getting disk info: {str(e)}")
                continue
        return disks

    def _get_motherboard_info(self) -> Dict[str, Any]:
        """Get motherboard information."""
        board = self.wmi.Win32_BaseBoard()[0]
        return {
            'manufacturer': board.Manufacturer,
            'product': board.Product,
            'serial': board.SerialNumber,
            'version': board.Version
        }

    def _get_graphics_info(self) -> List[Dict[str, Any]]:
        """Get graphics card information."""
        graphics = []
        for gpu in self.wmi.Win32_VideoController():
            try:
                graphics.append({
                    'name': gpu.Name,
                    'manufacturer': gpu.VideoProcessor,
                    'memory': gpu.AdapterRAM,
                    'resolution': f"{gpu.CurrentHorizontalResolution}x"
                                f"{gpu.CurrentVerticalResolution}",
                    'refresh_rate': gpu.CurrentRefreshRate,
                    'driver_version': gpu.DriverVersion
                })
            except Exception as e:
                self.logger.debug(f"Error getting GPU info: {str(e)}")
                continue
        return graphics

    def _get_bios_info(self) -> Dict[str, Any]:
        """Get BIOS information."""
        bios = self.wmi.Win32_BIOS()[0]
        return {
            'manufacturer': bios.Manufacturer,
            'version': bios.Version,
            'release_date': bios.ReleaseDate,
            'serial': bios.SerialNumber
        }

    def _get_windows_info(self) -> Dict[str, Any]:
        """Get detailed Windows information."""
        os = self.wmi.Win32_OperatingSystem()[0]
        return {
            'os_name': os.Caption,
            'version': os.Version,
            'build_number': os.BuildNumber,
            'serial': os.SerialNumber,
            'registered_user': os.RegisteredUser,
            'organization': os.Organization,
            'install_date': os.InstallDate,
            'last_boot': os.LastBootUpTime
        }

    def _get_uac_settings(self) -> Dict[str, Any]:
        """Get UAC settings."""
        try:
            key = winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE,
                r"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Policies\\System"
            )
            return {
                'enabled': bool(winreg.QueryValueEx(key, "EnableLUA")[0]),
                'consent_prompt': winreg.QueryValueEx(
                    key, "ConsentPromptBehaviorAdmin"
                )[0],
                'secure_desktop': bool(winreg.QueryValueEx(
                    key, "PromptOnSecureDesktop"
                )[0])
            }
        except Exception as e:
            self.logger.error(f"Error getting UAC settings: {str(e)}")
            return {}

    def _get_firewall_settings(self) -> Dict[str, Any]:
        """Get Windows Firewall settings."""
        firewall = {}
        try:
            for profile in self.wmi.Win32_FirewallSettings():
                firewall[profile.Name] = {
                    'enabled': profile.EnableFirewall,
                    'block_inbound': profile.BlockAllInboundTraffic,
                    'notifications': profile.NotifyOnListen
                }
        except Exception as e:
            self.logger.error(f"Error getting firewall settings: {str(e)}")
        return firewall

    def _get_antivirus_info(self) -> List[Dict[str, Any]]:
        """Get antivirus software information."""
        antivirus = []
        try:
            for product in self.wmi.Win32_Product():
                if any(av in product.Name.lower() for av in 
                      ['antivirus', 'security', 'defender']):
                    antivirus.append({
                        'name': product.Name,
                        'vendor': product.Vendor,
                        'version': product.Version,
                        'install_date': product.InstallDate
                    })
        except Exception as e:
            self.logger.error(f"Error getting antivirus info: {str(e)}")
        return antivirus

    def _get_windows_updates(self) -> List[Dict[str, Any]]:
        """Get Windows Update information."""
        updates = []
        try:
            for update in self.wmi.Win32_QuickFixEngineering():
                updates.append({
                    'id': update.HotFixID,
                    'description': update.Description,
                    'installed_on': update.InstalledOn,
                    'installed_by': update.InstalledBy
                })
        except Exception as e:
            self.logger.error(f"Error getting Windows updates: {str(e)}")
        return updates

    def _get_network_interfaces(self) -> List[Dict[str, Any]]:
        """Get network interface information."""
        interfaces = []
        for interface in self.wmi.Win32_NetworkAdapter(
            PhysicalAdapter=True
        ):
            try:
                config = interface.associators(
                    "Win32_NetworkAdapterConfiguration"
                )
                if config:
                    config = config[0]
                    interfaces.append({
                        'name': interface.Name,
                        'adapter_type': interface.AdapterType,
                        'mac_address': interface.MACAddress,
                        'manufacturer': interface.Manufacturer,
                        'speed': interface.Speed,
                        'ip_addresses': config.IPAddress,
                        'subnet_masks': config.IPSubnet,
                        'default_gateway': config.DefaultIPGateway,
                        'dns_servers': config.DNSServerSearchOrder
                    })
            except Exception as e:
                self.logger.debug(
                    f"Error getting network interface info: {str(e)}"
                )
                continue
        return interfaces

    def _get_dns_servers(self) -> List[str]:
        """Get DNS server information."""
        dns_servers = []
        try:
            for nic in self.wmi.Win32_NetworkAdapterConfiguration(
                IPEnabled=True
            ):
                if nic.DNSServerSearchOrder:
                    dns_servers.extend(nic.DNSServerSearchOrder)
        except Exception as e:
            self.logger.error(f"Error getting DNS servers: {str(e)}")
        return list(set(dns_servers))  # Remove duplicates

    def _get_network_shares(self) -> List[Dict[str, Any]]:
        """Get network share information."""
        shares = []
        resume = 0
        while True:
            try:
                share_list, total, resume = win32net.NetShareEnum(
                    None, 2, resume
                )
                shares.extend([{
                    'name': share['netname'],
                    'path': share['path'],
                    'type': share['type'],
                    'remark': share['remark']
                } for share in share_list])
                if not resume:
                    break
            except win32net.error:
                break
        return shares

    def _format_uptime(self, boot_time: float) -> str:
        """Format system uptime."""
        uptime = time.time() - boot_time
        days = int(uptime // 86400)
        hours = int((uptime % 86400) // 3600)
        minutes = int((uptime % 3600) // 60)
        return f"{days}d {hours}h {minutes}m"

    def format_size(self, size: int) -> str:
        """Format size in bytes to human-readable format."""
        for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
            if size < 1024.0:
                return f"{size:.2f} {unit}"
            size /= 1024.0
        return f"{size:.2f} PB"

    def display_results(self, data: Dict[str, Any], 
                       output_format: str = 'table',
                       sections: Optional[List[str]] = None):
        """Display system information in specified format."""
        if sections:
            filtered_data = {k: v for k, v in data.items() if k in sections}
        else:
            filtered_data = data
            
        if output_format == 'json':
            print(json.dumps(filtered_data, indent=4, default=str))
        else:
            self._display_table_format(filtered_data)

    def _display_table_format(self, data: Dict[str, Any]):
        """Display data in table format."""
        for section, info in data.items():
            print(f"\n=== {section.upper()} ===")
            
            if isinstance(info, dict):
                for key, value in info.items():
                    if isinstance(value, (dict, list)):
                        print(f"\n{key}:")
                        if isinstance(value, dict):
                            for k, v in value.items():
                                print(f"  {k}: {v}")
                        else:  # list
                            for item in value:
                                if isinstance(item, dict):
                                    for k, v in item.items():
                                        print(f"  {k}: {v}")
                                else:
                                    print(f"  {item}")
                    else:
                        print(f"{key}: {value}")
                        
            elif isinstance(info, list):
                if info and isinstance(info[0], dict):
                    headers = info[0].keys()
                    table_data = [[str(item.get(h, '')) for h in headers] 
                                for item in info]
                    print(tabulate(table_data, headers=headers, tablefmt='grid'))
                else:
                    for item in info:
                        print(item)

def main():
    parser = argparse.ArgumentParser(
        description='System Information Viewer - '
                   'Windows system information utility')
    parser.add_argument('--format', choices=['table', 'json'],
                       default='table',
                       help='Output format')
    parser.add_argument('--sections', nargs='+',
                       choices=['hardware', 'os', 'software', 'drivers', 
                              'security', 'network', 'users'],
                       help='Specific sections to display')
    
    args = parser.parse_args()
    
    # Setup logging
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s - %(levelname)s - %(message)s'
    )
    
    try:
        viewer = SystemInformationViewer()
        info = viewer.get_system_info()
        viewer.display_results(info, args.format, args.sections)
        
    except Exception as e:
        logging.error(f"An error occurred: {str(e)}")
        return 1
        
    return 0

if __name__ == "__main__":
    exit(main())
